@@ -8,17 +8,24 @@ use base qw(Exporter);
 use Carp qw(croak);
 use ALPM;
 
-my %_PKG_ATTRIBS = map { /^(?:get|has)_(.*)$/ ? ( $1 => $_ ) : ( $_ => $_ ) }
-    qw { get_filename get_name get_version get_desc get_url
-         get_builddate get_installdate get_packager get_md5sum
-         get_arch get_size get_isize get_reason get_licenses
-         get_groups get_depends get_optdepends get_conflicts
-         get_provides get_deltas get_replaces get_files get_backup
+my %_PKG_ATTRIBS = map { /^has_(.*)$/ ? ( $1 => $_ ) : ( $_ => $_ ) }
+    qw { filename name version desc url
+         builddate installdate packager md5sum
+         arch size isize reason licenses
+         groups depends optdepends conflicts
+         provides deltas replaces files backup
          has_scriptlet has_force download_size };
 
-sub get_attr
+
+#---HELPER METHOD---
+#  Usage   : my $name = $pkg->attr('name');
+#  Purpose : This is used by attribs().
+#  Params  : The name of the attribute.
+#  Returns : The attribute value.
+#-------------------
+sub _attr
 {
-    croak "Invalid arguments to get_attrib method" if (@_ != 2);
+    croak "Invalid arguments to attrib method" if (@_ != 2);
     my ($self, $attrib_name) = @_;
 
     croak qq{Unknown package attribute "$attrib_name"}
@@ -37,29 +44,21 @@ sub get_attr
     return $result;
 }
 
-sub get_attribs_ref
+sub attribs
 {
     my $self = shift;
 
-    return [ map { $self->get_attr($_) } @_ ]
+    return [ map { $self->_attr($_) } @_ ]
         if ( @_ > 0 );
 
-    my $attribs_ref;
-    for my $attrib_name ( keys %_PKG_ATTRIBS ) {
-        $attribs_ref->{$attrib_name} = $self->get_attr($attrib_name);
-    }
-    return $attribs_ref;
+    return map { ( $_ => $self->_attr($_) ) } keys %_PKG_ATTRIBS;
 }
 
-sub get_attribs
+sub attribs_ref
 {
     my $self = shift;
 
-    if ( @_ == 0 ) {
-        return %{$self->get_attribs_ref(@_)};
-    }
-
-    return @{$self->get_attribs_ref(@_)};
+    return ( @_ == 0 ? { $self->attribs(@_) } : [ $self->attribs(@_) ] );
 }
 
 1;
@@ -75,95 +74,94 @@ ALPM::Package - Class representing an alpm package
 
   # TMTOWTDI!
 
-  my $name = $perlpkg->get_name();
+  my $name = $perlpkg->name();
   print "$name rocks!\n";
 
-  print $perlpkg->get_attr('name'), " rocks!\n";
-
-  my %attrs = $perlpkg->get_attribs();
+  my %attrs = $perlpkg->attribs();
   print "$attrs{name} rocks!\n";
 
-  my $attrs_ref = $perlpkg->get_attribs_ref();
+  my $attrs_ref = $perlpkg->attribs_ref();
   print "attrs_ref->{name} rocks!\n";
 
   # Dependencies are given as array of hashrefs:
   print "$name depends on:\n";
-  for my $dep ( @{ $perlpkg->get_depends() } ) {
-      print "\t$dep->{name} $dep->{mod} $dep->{ver}\n";
+  for my $dep ( @{ $perlpkg->depends() } ) {
+      my @words = @{$dep}{'name', 'mod', 'ver'};
+      print "  @words\n";
   }
 
   # Others lists are arrayrefs of scalars:
   print "$name owns files:\n";
-  for my $file ( @{ $perlpkg->get_files() } ) {
-      print "\t$file\n";
-  }
+  print "  $_\n" foreach ( @{ $perlpkg->files } );
 
 =head1 DESCRIPTION
 
 This class is a wrapper for all of the C<alpm_pkg_...> C library functions
 of libalpm.  You retrieve the package from the database and you can then
 access its attributes.  Attributes are like L<ALPM>'s options.  There are
-many different ways to access them.  Yet you cannot modify a package
-using its methods.
+many different ways to access them.  You cannot modify a package.
 
 =head1 ATTRIBUTES
 
 There are three basic ways to access an attribute.  You can use the
 accessor method that is specific to an attribute, you can use the
-C<get_attr> method, or you can use the C<get_attribs> method.
+C<attr> method, or you can use the C<attribs> method.
 
 =head2 ATTRIBUTE ACCESSORS
 
 The accessors are named exactly the same as the C<alpm_pkg_get...>
 functions.  They are easy to use if you only want a few attributes.
 
+I have removed the get_ prefix on the accessors.  This is because you
+can't really I<set> anything so you should know it's a get anyways.
+
 =over
 
-=item * get_filename
+=item * filename
 
-=item * get_name
+=item * name
 
-=item * get_version
+=item * version
 
-=item * get_desc
+=item * desc
 
-=item * get_url
+=item * url
 
-=item * get_builddate
+=item * builddate
 
-=item * get_installdate
+=item * installdate
 
-=item * get_packager
+=item * packager
 
-=item * get_md5sum
+=item * md5sum
 
-=item * get_arch
+=item * arch
 
-=item * get_size
+=item * size
 
-=item * get_isize
+=item * isize
 
-=item * get_reason
+=item * reason
 
-=item * get_licenses
+=item * licenses
 
-=item * get_groups
+=item * groups
 
-=item * get_depends
+=item * depends
 
-=item * get_optdepends
+=item * optdepends
 
-=item * get_conflicts
+=item * conflicts
 
-=item * get_provides
+=item * provides
 
-=item * get_deltas
+=item * deltas
 
-=item * get_replaces
+=item * replaces
 
-=item * get_files
+=item * files
 
-=item * get_backup
+=item * backup
 
 =item * has_scriptlet
 
@@ -171,11 +169,13 @@ functions.  They are easy to use if you only want a few attributes.
 
 =item * download_size
 
+=item * changelog
+
 =back
 
 Attributes with plural names return an arrayref of strings.
 
-get_depends is different because it returns an arrayref of hashrefs
+depends is different because it returns an arrayref of hashrefs
 (an AoH).  The hash has the following key-value pairs:
 
   |------+----------------------------------------|
@@ -189,31 +189,21 @@ get_depends is different because it returns an arrayref of hashrefs
 
 =head2 PERLISH METHODS
 
-There are also more perlish ways to get attributes.  C<get_attr> is useful
-if you have the name of the attribute in a scalar.  C<get_attribs> is useful
-if you want to get all attributes at once in a hash, or many attributes
+There are also more ways to get attributes.  C<attribs> is useful if
+you want to get all attributes at once in a hash, or many attributes
 at once into a list or variables.
 
-=head2 get_attr
+=head2 attribs
 
-  Usage   : my $name = $pkg->get_attr('name');
-  Params  : The name of the attribute.
-            (use 'name' to call get_name, etc.
-             use 'scriplet' to call has_scriptlet, etc.
-             'download_size' is unchanged)
-  Returns : The attribute value.
-
-=head2 get_attribs
-
-  Usage   : my %attribs = $pkg->get_attribs();
-            my ($name, $desc) = $pkg->get_attribs('name', 'desc');
+  Usage   : my %attribs = $pkg->attribs();
+            my ($name, $desc) = $pkg->attribs('name', 'desc');
   Params  : If you specify attribute names, their values are returned as
             a list.  Otherwise, returns a hash of all attributes.
   Returns : Either a hash or a list.
 
-=head2 get_attribs_ref
+=head2 attribs_ref
 
-  This is the same as get_attribs, but it returns a hashref or
+  This is the same as attribs, but it returns a hashref or
   arrayref instead.
 
 =head1 SEE ALSO
@@ -222,7 +212,7 @@ L<ALPM>, L<ALPM::DB>
 
 =head1 AUTHOR
 
-Justin Davis, C<< <jrcd83 at gmail dot com> >>
+Justin Davis, C<< <jrcd83 at gmail> >>
 
 =head1 COPYRIGHT AND LICENSE
 
